@@ -126,6 +126,56 @@ pub trait AudioNode {
         Self::Outputs::USIZE
     }
 
+    /// Evaluate frequency response of `output` at `frequency` Hz.
+    /// Any linear response can be composed.
+    /// Return `None` if there is no response or it could not be calculated.
+    fn response(&self, output: usize, frequency: f64) -> Option<Complex64> {
+        assert!(output < self.outputs());
+        let mut input = new_signal_frame(self.inputs());
+        for i in 0..self.inputs() {
+            input[i] = Signal::Response(Complex64::new(1.0, 0.0), 0.0);
+        }
+        let response = self.route(&input, frequency);
+        match response[output] {
+            Signal::Response(rx, _) => Some(rx),
+            _ => None,
+        }
+    }
+
+    /// Evaluate frequency response of `output` in dB at `frequency Hz`.
+    /// Any linear response can be composed.
+    /// Return `None` if there is no response or it could not be calculated.
+    fn response_db(&self, output: usize, frequency: f64) -> Option<f64> {
+        assert!(output < self.outputs());
+        self.response(output, frequency).map(|r| amp_db(r.norm()))
+    }
+
+    /// Causal latency in (fractional) samples.
+    /// After a reset, we can discard this many samples from the output to avoid incurring a pre-delay.
+    /// The latency can depend on the sample rate and is allowed to change after `reset`.
+    fn latency(&self) -> Option<f64> {
+        if self.outputs() == 0 {
+            return None;
+        }
+        let mut input = new_signal_frame(self.inputs());
+        for i in 0..self.inputs() {
+            input[i] = Signal::Latency(0.0);
+        }
+        // The frequency argument can be anything as there are no responses to propagate,
+        // only latencies. Latencies are never promoted to responses during signal routing.
+        let response = self.route(&input, 1.0);
+        // Return the minimum latency.
+        let mut result: Option<f64> = None;
+        for output in 0..self.outputs() {
+            match (result, response[output]) {
+                (None, Signal::Latency(x)) => result = Some(x),
+                (Some(r), Signal::Latency(x)) => result = Some(r.min(x)),
+                _ => (),
+            }
+        }
+        result
+    }
+
     /// Retrieve the next mono sample from a generator.
     /// The node must have no inputs and exactly 1 output.
     #[inline]
@@ -249,65 +299,7 @@ impl<T: Float> AudioNode for Pass<T> {
     }
 }
 
-/// Pass through input unchanged.
-/// Latest input value can be queried as a read-only parameter.
-#[derive(Default)]
-pub struct Monitor<T> {
-    _marker: PhantomData<T>,
-    tag: Tag,
-    value: T,
-}
-
-impl<T: Float> Monitor<T> {
-    /// Create a new monitor node.
-    pub fn new(tag: Tag) -> Self {
-        Self {
-            _marker: PhantomData::default(),
-            tag,
-            value: T::zero(),
-        }
-    }
-}
-
-impl<T: Float> AudioNode for Monitor<T> {
-    const ID: u64 = 56;
-    type Sample = T;
-    type Inputs = U1;
-    type Outputs = U1;
-
-    #[inline]
-    fn tick(
-        &mut self,
-        input: &Frame<Self::Sample, Self::Inputs>,
-    ) -> Frame<Self::Sample, Self::Outputs> {
-        self.value = input[0];
-        *input
-    }
-
-    fn process(
-        &mut self,
-        size: usize,
-        input: &[&[Self::Sample]],
-        output: &mut [&mut [Self::Sample]],
-    ) {
-        self.value = input[0][size - 1];
-        output[0][..size].clone_from_slice(&input[0][..size]);
-    }
-
-    fn route(&self, input: &SignalFrame, _frequency: f64) -> SignalFrame {
-        input.clone()
-    }
-
-    fn get(&self, parameter: Tag) -> Option<f64> {
-        if self.tag == parameter {
-            Some(self.value.to_f64())
-        } else {
-            None
-        }
-    }
-}
-
-/// Present time as a parameter. The time can be set as well.
+/// Present time as a parameter.
 #[derive(Default)]
 pub struct Timer<T> {
     _marker: PhantomData<T>,
@@ -359,12 +351,6 @@ impl<T: Float> AudioNode for Timer<T> {
         self.time += size as f64 * self.sample_duration;
     }
 
-    fn set(&mut self, parameter: Tag, value: f64) {
-        if self.tag == parameter {
-            self.time = value;
-        }
-    }
-
     fn get(&self, parameter: Tag) -> Option<f64> {
         if self.tag == parameter {
             Some(self.time)
