@@ -26,6 +26,8 @@ pub use super::*;
 
 pub use num_complex::Complex32;
 
+use std::sync::Arc;
+
 // Combinator environment.
 // We like to define all kinds of useful functions here.
 
@@ -1472,13 +1474,21 @@ pub fn pulse() -> An<super::prelude::PulseWave<f32>> {
 /// Play back a channel of a Wave64.
 /// Optional loop point is the index to jump to at the end of the wave.
 /// - Output 0: wave
-pub fn wave64(wave: &Wave64, channel: usize, loop_point: Option<usize>) -> An<Wave64Player<f32>> {
+pub fn wave64(
+    wave: Arc<Wave64>,
+    channel: usize,
+    loop_point: Option<usize>,
+) -> An<Wave64Player<f32>> {
     An(Wave64Player::new(wave, channel, loop_point))
 }
 
 /// Play back a channel of a Wave32.
 /// Optional loop point is the index to jump to at the end of the wave.
 /// - Output 0: wave
-pub fn wave32(wave: &Wave32, channel: usize, loop_point: Option<usize>) -> An<Wave32Player<f32>> {
+pub fn wave32(
+    wave: Arc<Wave32>,
+    channel: usize,
+    loop_point: Option<usize>,
+) -> An<Wave32Player<f32>> {
     An(Wave32Player::new(wave, channel, loop_point))
 }
