@@ -249,6 +249,7 @@ pub mod hacker;
 pub mod hacker32;
 pub mod math;
 pub mod moog;
+pub mod net;
 pub mod noise;
 pub mod oscillator;
 pub mod oversample;
@@ -259,6 +260,7 @@ pub mod scale;
 pub mod sequencer;
 pub mod shape;
 pub mod signal;
+pub mod sound;
 pub mod svf;
 pub mod wave;
 pub mod wavetable;
